// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 9th April 2007
//
// Purpose:       Demo shows how to use HEL's Background Text System
//                to print text on a tile based background with
//                dynamic tile reloading enabled.
//
//                See makefile for graphic conversion details.
//
//                I recommend to take a look at the following
//                demo projects first, since this one is based
//                on them: MapDynamicTileReloading, BgText
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------

// The background number the 
// background Text System uses.
#define BGNO        0


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------

// A buffer of memory which the Background Text
// System uses to manage internal states.
// It must not be modified from outside as long as
// any Background Text System is running.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_BgTextSystemMemory[HEL_SUBSYSTEM_BGTEXT_REQUIREDMEMORY];

// g_CharLUT will hold offsets to each character
// in the map data. This array basically functions
// as a lookup table. It must be 256 elements big,
// which equals the maximum number of a characters
// in ASCII (0..255).
// This buffer is only necessary when you pass
// BGTEXT_FLAGS_GENERATELUT to hel_BgTextCreate.
// Please see documentation for details.
u16 ATTR_EWRAM g_CharLUT[256];

// That's the order of characters how they
// appear on the source graphic.
// See hel/demos/sharedmedia/charset8x8.bmp
const char CHARORDER[] =
" BCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmn"\
"opqrstuvwxyz0123456789?,.-;:A#^+*@"\
"!\"%\%&/()=?|\\<>[]{}";

const char* const TEXTS[] =
{
	"You are watching...        ",
		"the background text demo...",
		"with dynamic tile ...      ",
		"reloading feature enabled! ",
		"This means only tiles...   ",
		"which are actually used... ",
		"are loaded to video ram!   ",
		"                  ===> END "
};

// -----------------------------------------------------------------------------
// Function Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define NUMBER_TILES_ROM    (102)
#define NUMBER_TILES_RAM    (21*31)

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_TileSystemBuffer[HEL_SUBSYSTEM_TILE_REQUIREDMEMORY];
u16 ATTR_EWRAM g_BufferRom[NUMBER_TILES_ROM];
u16 ATTR_EWRAM g_BufferRam[NUMBER_TILES_RAM*3];


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
	ham_Init();
	hel_SysSetPrefetch(TRUE);
	hel_BgSetMode(0);


	// Initialize the Background Text System.
	// The g_BgTextSystemMemory array is used by the
	// Background Text System to manage internal states
	// and must not be modified from outside as long as
	// any Background Text System is running.
	hel_BgTextInit((void*)g_BgTextSystemMemory);

	// Initialize the Tile System
	hel_TileInit((void*)g_TileSystemBuffer);


	// Assign the tileset to a Tile System
	hel_TileCreate(BGNO, ResData(RES_CHARSET8X8_RAW), NUMBER_TILES_ROM, g_BufferRom, NUMBER_TILES_RAM, g_BufferRam, COLORS_16, 1, TRUE);

	// Here we use a function from HAM to
	// initialize an empty map in video ram.
	ham_bg[BGNO].mi = ham_InitMapEmptySet(MAP_SIZE_32X32, FALSE);


	// Create a Background Text System
	hel_BgTextCreate(
		BGNO,   // BgNo
		1,      // Width of one character specified in tiles
		1,      // Height of one character specified in tiles
		ResData(RES_CHARSET8X8_MAP),    // Pointer to character map
		CHARORDER,      // Pointer to array that reflects the order of characters on source graphic
		g_CharLUT,      // Pointer to buffer that receives the generated lookup table
		BGTEXT_FLAGS_GENERATELUT | BGTEXT_FLAGS_DYNAMICTILERELOADING);  // Flags that control creation and print behaviour


	// Load palette data to Vram
	hel_PalBgLoad16(ResData16(RES_CHARSET8X8_PAL), 1);

	// Initialize and show background
	ham_InitBg(BGNO, TRUE, 0, FALSE);

	// Start interrupt
	hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);


	int currentText=0;
	int delay=120;

	// infinite loop to hold
	// the application alive
	for(;;)
	{
		if(++delay > 120)
		{
			hel_BgTextPrint(BGNO,
				1,      // X
				1,      // Y
				0,      // Flags, must be zero at this time
				TEXTS[currentText&7]
				);

				delay = 0;
				currentText++;
		}

		hel_SwiVBlankIntrWait();
	}

	return 0;
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
	hel_IntrAcknowledge(INT_TYPE_VBL);
}