// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use the map-system.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------

// Reserve memory for the Map-System. This buffer
// is used to manage internal states of the Map-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void ScrollMap();
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    
    // Initialize the Map-System. If you do not initialize
    // the Map-System before calling any Map related function,
    // the program behaviour is undefined.
    hel_MapInit(g_MapSystemBuffer);
    
    ham_SetBgMode(0);
    
    // Load Tileset into VideoRAM
    ham_bg[0].ti = ham_InitTileSet(ResData(RES_RACETRACK_RAW), RES_RACETRACK_RAW_SIZE16, 1, TRUE);
    
    // Create a map for background 0
    hel_MapCreate(0,        // Background number
                  1024/8,   // width in tiles
                  1024/8,   // height in tiles
                  ResData(RES_RACETRACK_MAP),   // Pointer to source MapData
                  sizeof(u16),                  // DataTypeSize of one element from Source MapData
                  MAP_FLAGS_DEFAULT);           // Flags to control map behaviour
    
    // Load Background Palette
    hel_PalBgLoad256(ResData(RES_RACETRACK_PAL));
    
    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    // Infinite loop to keep
    // the application alive
    for(;;)
    {
      hel_PadCapture();
      ScrollMap();
          
      hel_SwiVBlankIntrWait();
    }
    
    return 0;
}


// -----------------------------------------------------------------------------
// Queries the Pad and scrolls the map depending on the pad input
// -----------------------------------------------------------------------------
void ScrollMap()
{
   #define SCROLL_SPEED_X     FIXED_FROMFLOAT(3.0f)
   #define SCROLL_SPEED_Y     FIXED_FROMFLOAT(2.0f)

   TPointSfp32 ScrollBy={0, 0};

   // Query Pad to see in what direction
   // the map should be scrolled
   if(hel_PadQuery()->Held.Left)
   {
      ScrollBy.X -= SCROLL_SPEED_X;
   }
   else
   if(hel_PadQuery()->Held.Right)
   {
      ScrollBy.X += SCROLL_SPEED_X;
   }

   if(hel_PadQuery()->Held.Down)
   {
      ScrollBy.Y += SCROLL_SPEED_Y;
   }
   else
   if(hel_PadQuery()->Held.Up)
   {
      ScrollBy.Y -= SCROLL_SPEED_Y;
   }

   // Now Scroll the map!
   hel_MapScroll(0, ScrollBy.X, ScrollBy.Y);
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	hel_IntrAcknowledge(INT_TYPE_VBL);
}

