// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use a VirtualMap,
//                as an invisible collision layer.
//
// Notes:         Levelgraphics borrowed from Nessie (http://nessie.gbadev.org)
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Structures
// -----------------------------------------------------------------------------
typedef struct THero
{
  THandle  ObjNo;
  sfp32 X, Y;
}THero;


// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define HERO_WIDTH      FIXED_FROMINT(8)
#define HERO_HEIGHT     FIXED_FROMINT(8)
#define COLLISION_NONE  1

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];
THero ATTR_EWRAM g_Hero;


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    
    // initialize Object-System
    hel_ObjInit(g_ObjSystemBuffer);
    
    // initialize Map-System
    hel_MapInit(g_MapSystemBuffer);
    
    ham_SetBgMode(0);
    
    // Init 16 Color Tileset
    ham_bg[0].ti = ham_InitTileSet(ResData(RES_WORLD2_LAYER0_RAW), RES_WORLD2_LAYER0_RAW_SIZE16, 0, TRUE);

    // Init Map and CollisionMap
    hel_MapCreate(0, 30, 20, ResData(RES_WORLD2_LAYER0_MAP), sizeof(u16), MAP_FLAGS_DEFAULT);
    hel_MapCreate(BG_VIRTUAL, 30, 20, ResData(RES_WORLD2_COLLISION_MAP), sizeof(u16), MAP_FLAGS_DEFAULT);

    // Create the Hero Sprite. It's actually
    // only a small Circle!
    g_Hero.X = FIXED_FROMINT(17);
    g_Hero.Y = FIXED_FROMINT(17);
    g_Hero.ObjNo = hel_ObjCreate256(ResData(RES_SMALLCIRCLE_RAW), OBJ_SHAPE_SQUARE, 0, COLORS_256, 241, 161);
    
    // Load Background Palette
    hel_PalBgLoad16(ResData16(RES_WORLD2_LAYER0_PAL), 0);
    
    // Load Object Palette
    hel_PalObjLoad256(ResData16(RES_SMALLCIRCLE_PAL));
    
    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    
    for(;;)
    {
        TPointSfp32 MoveBy={0,0};
        sfp32 AdjustX=0;
        sfp32 AdjustY=0;
        
        // Read current pad state
        hel_PadCapture();

        if(hel_PadQuery()->Held.Down)
        {
            MoveBy.Y += FIXED_ONE;
            AdjustY = HERO_HEIGHT;
        }
        else
        if(hel_PadQuery()->Held.Up)
        {
            MoveBy.Y -= FIXED_ONE;
        }

        if(hel_PadQuery()->Held.Left)
        {
            MoveBy.X -= FIXED_ONE;
        }
        else
        if(hel_PadQuery()->Held.Right)
        {
            MoveBy.X += FIXED_ONE;
            AdjustX = HERO_WIDTH;
        }


        // Get TileNumber of position where Hero wants to go to
        u16 TileNo = *(u16*)hel_MapGetTilePtrAt(BG_VIRTUAL, g_Hero.X + AdjustX + MoveBy.X, g_Hero.Y + AdjustY + MoveBy.Y);
        
        // Only set new position when
        // there is no collision!
        if(COLLISION_NONE == TileNo)
        {
          g_Hero.X += MoveBy.X;
          g_Hero.Y += MoveBy.Y;
        }

        hel_ObjSetXY(g_Hero.ObjNo, FIXED_TOINT(g_Hero.X), FIXED_TOINT(g_Hero.Y));

        hel_SwiVBlankIntrWait();
    }

    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
  hel_ObjTransmit();
  
  hel_IntrAcknowledge(INT_TYPE_VBL);
}

