// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use the map-system with dynamic tile
//                reloading enabled.
//
//                You should have read the following samples before:
//                   Map
//
//
//                HEL's tilesystem supports 4 and 8 bit graphics, but supports
//                only non rotation maps. It expects 16bit tilenumbers,
//                which means you cannot use flipped tiles.
//
//                If you use gfx2gba to convert the graphics, pass -F to
//                avoid flipped tiles. Here is how the sample-image is converted:
//                gfx2gba -t8 -m -F -fsrc -ogfx -pvampire.pal ../SharedMedia/gfx/vampire.bmp
//
//                4bit graphic notes:
//                  gfx2gba v0.14 has no option to prevent it from storing the palettenumber
//                  in each tile when dealing with 4bit graphics. This results in tilecorruption
//                  when using HEL's tilesystem. However, there is a workaround. Just set the
//                  palettebank to 0 (zero) when converting the graphics. gfx2gba will not set the
//                  palettebank bits in this case.
//
//
//                Output from gfx2gba:
//
//                =======================
//                gfx2gba Converter v0.14
//                -----------------------
//                (C) 2oo1-2oo2 [TRiNiTY]
//                =======================
//
//                Reading: ../SharedMedia/gfx/vampire.bmp (800x600 pixel, 256 colors)
//                Number of tiles before optimization: 7500
//                Number of tiles after  optimization: 7438
//
//                ***WARNING: number of tiles (7438) exceeds 1024!
//                Saving tiled bitmap data to: gfx\vampire.raw.c ... ok
//                Saving map data to: gfx\vampire.map.c ... ok
//
//                Saving masterpalette to..: gfx\vampire.pal.c ... ok
//
//                Total files read & converted.:   1
//                Colors used before converting: 255
//                Colors used after converting.: 256
//                Colors saved.................:  -1
//
//
//
//                "Number of tiles" becomes important later in this example.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
// This equals "Number of tiles after  optimization"
#define NUMBER_TILES_ROM    (7438)    


// Because the image has so many unique tiles, there
// can be in worst case 20*30 tiles on the screen at
// the same time. We advance both values by 1 to have
// some buffer to precache tiles.
//
// Notes:
//  if you have a huge amount of tiles, please keep in
//  mind the vram can only contain
//  a maximum of 1024 tiles at the same time. I just wanted
//  to mention it because this implies it makes no sense to set
//  NUMBER_TILES_RAM to a value greater than 1024
#define NUMBER_TILES_RAM    (21*31)

// Size of map in tiles
#define VAMPIRE_WIDTH       (800/8)
#define VAMPIRE_HEIGHT      (600/8)

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];

// This buffer is used to resolve Rom to Ram tilenumbers.
u16 ATTR_EWRAM g_BufferRom[NUMBER_TILES_ROM];

// This buffer is to resolve Ram to Rom tilenumbers,
// reference counting and a "stack" to pop and push
// tileslots.
u16 ATTR_EWRAM g_BufferRam[NUMBER_TILES_RAM*3];


// Reserve memory for the Tile-System. This buffer
// is used to manage internal states of the Tile-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_TileSystemBuffer[HEL_SUBSYSTEM_TILE_REQUIREDMEMORY];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void ScrollMap();
void VBLInterruptHandler(void);

#define PROFILE;
#ifdef PROFILE
  #include "Profiler.h"
  PROFILER_INIT;
#endif


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();

    // Initialize Map and Tile-System
    hel_MapInit(g_MapSystemBuffer);
    hel_TileInit(g_TileSystemBuffer);

    ham_SetBgMode(0);
    
    // Init Tile-System for Background 0
    hel_TileCreate(0,                // BgNo
                 ResData(RES_VAMPIRE_RAW),    // Source tiledata
                 NUMBER_TILES_ROM, // Number of tiles in ROM
                 g_BufferRom,      // BufferA
                 NUMBER_TILES_RAM, // Maximum number of tiles in RAM
                 g_BufferRam,      // BufferB
                 1,                // Tileset colormode (0=16colors, 1=256colors)
                 0,                // Palettenumber in 16color mode (0..15)
                 TRUE);            // CBB only mode
                                
    // Init Map-System for Background 0
    hel_MapCreate(0,                         // BgNo
                  VAMPIRE_WIDTH,             // MapWidth in tiles
                  VAMPIRE_HEIGHT,            // MapHeight in tiles
                  ResData(RES_VAMPIRE_MAP),  // Source MapData
                  sizeof(u16),               // DataTypeSize
                  MAP_FLAGS_DEFAULT | MAP_FLAGS_DYNAMICTILERELOADING); // Flags to control map behaviour

    // Load Background Palette
    hel_PalBgLoad256(ResData(RES_VAMPIRE_PAL));
    
    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);

    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
                       
   
    
    // Infinite loop
    for(;;)
    {
        hel_PadCapture();
        ScrollMap();
        
        hel_SwiVBlankIntrWait();
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// Queries the Pad and scrolls the map depending on the pad input
// -----------------------------------------------------------------------------
void ScrollMap()
{
   #define SCROLL_SPEED_X     FIXED_FROMFLOAT(3.0f)
   #define SCROLL_SPEED_Y     FIXED_FROMFLOAT(2.0f)

   TPointSfp32 ScrollBy={0, 0};

   // Query Pad to see in what direction
   // the map should be scrolled
   if(hel_PadQuery()->Held.Left)
   {
      ScrollBy.X -= SCROLL_SPEED_X;
   }
   else
   if(hel_PadQuery()->Held.Right)
   {
      ScrollBy.X += SCROLL_SPEED_X;
   }

   if(hel_PadQuery()->Held.Down)
   {
      ScrollBy.Y += SCROLL_SPEED_Y;
   }
   else
   if(hel_PadQuery()->Held.Up)
   {
      ScrollBy.Y -= SCROLL_SPEED_Y;
   }

#ifdef PROFILE
  PROFILER_START;
#endif

   // Now Scroll the map!
   hel_MapScroll(0, ScrollBy.X, ScrollBy.Y);

#ifdef PROFILE
  PROFILER_END;
#endif
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	hel_IntrAcknowledge(INT_TYPE_VBL);
}

