// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 22th April 2006
//
// Purpose:       Demo shows how to use the hel_MapGetTilePtrAt function
//                to receive the tilenumber from a specific coordinate.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"



// -----------------------------------------------------------------------------
// Structures
// -----------------------------------------------------------------------------
typedef struct _SCursor
{
  THandle hObj; // Object Handle
  sfp32   X;    // Cursor horizontal position in fixed point format
  sfp32   Y;    // Cursor vertical position in fixed point format
} SCursor;


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------

// Reserve memory for the Map-System. This buffer
// is used to manage internal states of the Map-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];

// Reserve memory for the Obj-System. This buffer
// is used to manage internal states of the Obj-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];

// Holds information about the cursor
// we can move around.
SCursor ATTR_EWRAM g_Cursor;


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void CreateCursor(void);
void UpdateCursor(void);
void DisplayInfo(void);
void ScrollMap(void);
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    
    // Initialize the Map-System. If you do not initialize
    // the Map-System before calling any Map related function,
    // the program behaviour is undefined.
    hel_MapInit(g_MapSystemBuffer);
    
    // Initialize Object-System
    hel_ObjInit(g_ObjSystemBuffer);
    
    // Set background mode
    hel_BgSetMode(0);

    // Init HAM's debug text
    ham_InitText(1);
    ham_SetTextCol(4, 0);
    
    // Load Tileset into VideoRAM
    ham_bg[0].ti = ham_InitTileSet(ResData(RES_WRAPTILEPICK_RAW), RES_WRAPTILEPICK_RAW_SIZE16, 1, TRUE);
    
    // Create a map for background 0
    hel_MapCreate(0,        // Background number
                  392/8,   // width in tiles
                  408/8,   // height in tiles
                  ResData(RES_WRAPTILEPICK_MAP),   // Pointer to source MapData
                  sizeof(u16),                  // DataTypeSize of one element from Source MapData
                  MAP_FLAGS_DEFAULT);           // Flags to control map behaviour
    
    // Turn off bounds checking, so the map
    // is able to wrap around
    hel_MapSetBounds(0,
                     FIXED_FROMINT(0), // Left
                     FIXED_FROMINT(0), // Top
                     FIXED_FROMINT(SCREEN_DISPLAY_WIDTH), // Right
                     FIXED_FROMINT(SCREEN_DISPLAY_HEIGHT), // Bottom
                     0); // Flags

    // Create cursor object
    CreateCursor();
    
    // Load Palettes
    hel_PalBgLoad256(ResData(RES_BG_PAL));
    hel_PalObjLoad256(ResData16(RES_OBJ_PAL));
        
    // Turn on Background
    ham_InitBg(0, TRUE, 1, FALSE);
    hel_BgSetPrio(1, 0);
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    // display instruction text
    ham_DrawText(1, 1, "d-pad to scroll map");
    ham_DrawText(1, 2, "d-pad and -a- to move cursor");
    
    ham_DrawText(1, 4, "black= 0");
    ham_DrawText(1, 5, "red  = 1");
    ham_DrawText(1, 6, "green= 2");
    ham_DrawText(1, 7, "blue = 3");
    ham_DrawText(1, 8, "white= 4");
    
    // Infinite loop to keep
    // the application alive
    for(;;)
    {
      hel_PadCapture();
      
      UpdateCursor();
      ScrollMap();
          
      DisplayInfo();
      
      hel_SwiVBlankIntrWait();
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// Display tile number from cursor position
// -----------------------------------------------------------------------------
void DisplayInfo(void)
{
  
  // display current map position
  sfp32 MapPosX;
  sfp32 MapPosY;
  
  hel_MapGetPosition(0, &MapPosX, &MapPosY);
  ham_DrawText(1, 15, "map    x:%d, y=%d ", FIXED_TOINT(MapPosX), FIXED_TOINT(MapPosY));
  
  
  // display current cursor position
  ham_DrawText(1, 16, "cursor x:%d, y=%d ", FIXED_TOINT(g_Cursor.X), FIXED_TOINT(g_Cursor.Y));
  
  
  // display tile number at current cursor position
  u16 Tile       = *(u16*)hel_MapGetTilePtrAt(0, g_Cursor.X, g_Cursor.Y);
  u16 TileNumber = Tile & 1023;

  ham_DrawText(1, 18, "Tilenumber at cursor:%d   ", TileNumber);
}

// -----------------------------------------------------------------------------
// Creates the cursor sprite
// -----------------------------------------------------------------------------
void CreateCursor(void)
{
  // Create an object!
  g_Cursor.hObj = hel_ObjCreate(
                     ResData(RES_CURSOR16X16_RAW), // Pointer to source graphic
                     OBJ_SHAPE_SQUARE,       // Obj Shape
                     1,                      // Obj Size, 1 means 16x16 pixels, if Shape is set to SQUARE
                     OBJ_MODE_NORMAL,        // Obj Mode
                     COLORS_256,             // Use 256 color mode
                     0,                      // Palette number. Only neccessary in 16 color mode
                     FALSE,                  // Don't use mosaic
                     FALSE,                  // Don't flip the sprite horizontally
                     FALSE,                  // Don't flip the object vertically
                     0,                      // Priority against background. 0=higest
                     FALSE,                  // Don't make the object double-sized
                     241,                    // Destination horzintal screen coordinate in pixels
                     161                     // Destination vertical screen coordinate in pixels
                  );
                  
  // Set cursor initial position
  g_Cursor.X = FIXED_FROMINT(120);
  g_Cursor.Y = FIXED_FROMINT(80);
}

// -----------------------------------------------------------------------------
// Queries the Pad and moves the cursor depending on the pad input
// -----------------------------------------------------------------------------
void UpdateCursor(void)
{
   #define SCROLL_SPEED_X     FIXED_FROMFLOAT(1.5f)
   #define SCROLL_SPEED_Y     FIXED_FROMFLOAT(1.5f)

   TPointSfp32 ScrollBy={0, 0};

   // Only scroll map when -A- button
   // is held, because otherwise we
   // want to scroll the map.
   if(hel_PadQuery()->Held.A)
   {
     // Query Pad to see in what direction
     // the map should be scrolled
     if(hel_PadQuery()->Held.Left)
     {
        ScrollBy.X -= SCROLL_SPEED_X;
     }
     else
     if(hel_PadQuery()->Held.Right)
     {
        ScrollBy.X += SCROLL_SPEED_X;
     }

     if(hel_PadQuery()->Held.Down)
     {
        ScrollBy.Y += SCROLL_SPEED_Y;
     }
     else
     if(hel_PadQuery()->Held.Up)
     {
        ScrollBy.Y -= SCROLL_SPEED_Y;
     }
     
     // Update Cursor Position
     g_Cursor.X += ScrollBy.X;
     g_Cursor.Y += ScrollBy.Y;
     
     // Limit the Cursor Position to the screen
     g_Cursor.X = LIMIT(g_Cursor.X, 0, FIXED_FROMINT(SCREEN_DISPLAY_WIDTH  - 9));
     g_Cursor.Y = LIMIT(g_Cursor.Y, 0, FIXED_FROMINT(SCREEN_DISPLAY_HEIGHT - 9));
   }
   
   // Set position in HEL
   hel_ObjSetXY(g_Cursor.hObj, FIXED_TOINT(g_Cursor.X), FIXED_TOINT(g_Cursor.Y));
   
   
   #undef SCROLL_SPEED_X
   #undef SCROLL_SPEED_Y
}

// -----------------------------------------------------------------------------
// Queries the Pad and scrolls the map depending on the pad input
// -----------------------------------------------------------------------------
void ScrollMap()
{
   #define SCROLL_SPEED_X     FIXED_FROMFLOAT(3.0f)
   #define SCROLL_SPEED_Y     FIXED_FROMFLOAT(2.0f)

   TPointSfp32 ScrollBy={0, 0};

   // Only scroll map when -A- button
   // is not held, because in this
   // case we scroll the cursor.
   if(!hel_PadQuery()->Held.A)
   {
     // Query Pad to see in what direction
     // the map should be scrolled
     if(hel_PadQuery()->Held.Left)
     {
        ScrollBy.X -= SCROLL_SPEED_X;
     }
     else
     if(hel_PadQuery()->Held.Right)
     {
        ScrollBy.X += SCROLL_SPEED_X;
     }

     if(hel_PadQuery()->Held.Down)
     {
        ScrollBy.Y += SCROLL_SPEED_Y;
     }
     else
     if(hel_PadQuery()->Held.Up)
     {
        ScrollBy.Y -= SCROLL_SPEED_Y;
     }

     // Now Scroll the map!
     hel_MapScroll(0, ScrollBy.X, ScrollBy.Y);
  }
  
  #undef SCROLL_SPEED_X
  #undef SCROLL_SPEED_Y
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
  hel_ObjTransmit();
  
	hel_IntrAcknowledge(INT_TYPE_VBL);
}

