// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 8th April 2007
//
// Purpose:       Demo shows how to use the Map-System with Parallax Scrolling
//                and Palette blending. The more you scroll to the right,
//                the more the Palette blends to its target.
//
// Notes:         Levelgraphics borrowed from Nessie (http://nessie.gbadev.org)
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define RATIO_LAYER_0_X     FIXED_FROMFLOAT(2.6f)
#define RATIO_LAYER_0_Y     FIXED_FROMFLOAT(2.6f)

#define RATIO_LAYER_1_X     FIXED_FROMFLOAT(1.3f)
#define RATIO_LAYER_1_Y     FIXED_FROMFLOAT(1.3f)

#define RATIO_LAYER_2_X     FIXED_FROMFLOAT(0.265f)
#define RATIO_LAYER_2_Y     FIXED_FROMFLOAT(0.44f)

#define LAYER0_WIDTH_IN_TILES	128
#define LAYER0_WIDTH			(8*LAYER0_WIDTH_IN_TILES)

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void ScrollMap();
void BlendPalette();
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    
    // Initialize the Map-System. If you do not initialize
    // the Map-System before calling any Map related function,
    // the program behaviour is undefined.
    hel_MapInit(g_MapSystemBuffer);
    
    ham_SetBgMode(0);
    
    // Create/Init Background 0
    
    // Load palette data to Vram
    hel_PalBgLoad16(ResData16(RES_WORLD1_LAYER0_PAL), 0);
    
    // Initialize a new tileset using HAM
	// This function copies the whole tileset into video ram
    ham_bg[0].ti = ham_InitTileSet(ResData(RES_WORLD1_LAYER0_RAW), RES_WORLD1_LAYER0_RAW_SIZE16, 0, TRUE);
    
    // Create a map
    hel_MapCreate(0, 128, 32, ResData(RES_WORLD1_LAYER0_MAP), sizeof(u16), MAP_FLAGS_DEFAULT);
    
    // Jump to the bottom left edge of the map
    hel_MapJumpTo(0, MAP_JUMPTO_LEFT | MAP_JUMPTO_BOTTOM);
    
    
    
    // Create/Init Background 1
    hel_PalBgLoad16(ResData16(RES_WORLD1_LAYER1_PAL), 1);
    ham_bg[1].ti = ham_InitTileSet(ResData(RES_WORLD1_LAYER1_RAW), RES_WORLD1_LAYER1_RAW_SIZE16, 0, TRUE);
    hel_MapCreate(1,  79, 26, ResData(RES_WORLD1_LAYER1_MAP), sizeof(u16), MAP_FLAGS_DEFAULT);
    hel_MapJumpTo(1, MAP_JUMPTO_LEFT | MAP_JUMPTO_BOTTOM);
    
    
    // Create/Init Background 2
    hel_PalBgLoad16(ResData16(RES_WORLD1_LAYER2_PAL), 2);
    ham_bg[2].ti = ham_InitTileSet(ResData(RES_WORLD1_LAYER2_RAW), RES_WORLD1_LAYER2_RAW_SIZE16, 0, TRUE);
    hel_MapCreate(2,  40, 22, ResData(RES_WORLD1_LAYER2_MAP), sizeof(u16), MAP_FLAGS_DEFAULT);
    hel_MapJumpTo(2, MAP_JUMPTO_LEFT | MAP_JUMPTO_BOTTOM);

    
    // Turn on backgrounds now, so
    // everything becomes visible
    ham_InitBg(0, TRUE, 0, FALSE);
    ham_InitBg(1, TRUE, 1, FALSE);
    ham_InitBg(2, TRUE, 2, FALSE);
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    
    for(;;)
    {
        hel_PadCapture();
        ScrollMap();
		BlendPalette();
        

        hel_SwiVBlankIntrWait();
    }

    return 0;
}

// -----------------------------------------------------------------------------
// Blend palette depending on the horizontal position of layer0
// -----------------------------------------------------------------------------
void BlendPalette()
{
	sfp32 x, y;							// receives the current position of layer 0
	sfp32 currentBlendWeight;			// received the current blending weight
	const int MAXIMUM_BLENDWEIGHT = 32; // maximum blending weight
	const int TARGET_PALETTEBANK  = 2;	// target palette bank

	// reciprocal of the maximum x position.
	sfp32 recipMaxX = FIXED_FROMFLOAT(1.0f / (LAYER0_WIDTH - SCREEN_DISPLAY_WIDTH));

	// get current position of layer 0
	hel_MapGetPosition(0, &x, &y);

	// compute blending weight depending on the current x position
	currentBlendWeight = FIXED_MUL(x, recipMaxX) * MAXIMUM_BLENDWEIGHT;

	// blend palettes and store result into palette video ram
	hel_PalBgBlend16(
		ResData16(RES_WORLD1_LAYER2_PAL),			// PaletteA
		ResData16(RES_WORLD1_LAYER2_EVENING_PAL),	// PaletteB
		TARGET_PALETTEBANK,							// Target PaletteBank
		FIXED_TOINT(currentBlendWeight)				// BlendWeight
		);
}

// -----------------------------------------------------------------------------
// Queries the Pad and scrolls the map depending on the pad input
//
// Slightly different approach to scroll maps than other examples from HEL.
// We have to scroll three layers, each with a different ratio. The ScrollBy
// variables just gives information in what direction to scroll, we
// use it to pass the corresponding scroll ratio to hel_MapScroll for each
// individual map layer
// -----------------------------------------------------------------------------
void ScrollMap()
{
   TPointSfp32 ScrollBy={0, 0};

   // Query Pad to see in what direction
   // the map should be scrolled
   if(hel_PadQuery()->Held.Left)
   {
      ScrollBy.X -= 1;
   }
   else
   if(hel_PadQuery()->Held.Right)
   {
      ScrollBy.X += 1;
   }

   if(hel_PadQuery()->Held.Down)
   {
      ScrollBy.Y += 1;
   }
   else
   if(hel_PadQuery()->Held.Up)
   {
      ScrollBy.Y -= 1;
   }

   // Now Scroll the layers!
   hel_MapScroll(0, RATIO_LAYER_0_X*ScrollBy.X, RATIO_LAYER_0_Y*ScrollBy.Y);
   hel_MapScroll(1, RATIO_LAYER_1_X*ScrollBy.X, RATIO_LAYER_1_Y*ScrollBy.Y);
   hel_MapScroll(2, RATIO_LAYER_2_X*ScrollBy.X, RATIO_LAYER_2_Y*ScrollBy.Y);
}


// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	hel_IntrAcknowledge(INT_TYPE_VBL);
}

