// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 29th April 2006
//
// Purpose:       Demo shows how to set a map to a given position.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Global Variables and Constants
// -----------------------------------------------------------------------------

// Reserve memory for the Map-System. This buffer
// is used to manage internal states of the Map-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];

// These markers have to appear at the top/left
// position. Check the graphic:
// hel/demos/SharedMedia/gfx/setpos.bmp
#define MAPMARKER_COUNT   10
const TPointSfp32 MAPMARKER[MAPMARKER_COUNT] =
{
  { FIXED_FROMINT(0),    FIXED_FROMINT(0)   },
  { FIXED_FROMINT(401),  FIXED_FROMINT(201) },
  { FIXED_FROMINT(1174), FIXED_FROMINT(60)  },
  { FIXED_FROMINT(1579), FIXED_FROMINT(165) },
  { FIXED_FROMINT(1824), FIXED_FROMINT(506) },
  { FIXED_FROMINT(1596), FIXED_FROMINT(847) },
  { FIXED_FROMINT(1080), FIXED_FROMINT(967) },
  { FIXED_FROMINT(890),  FIXED_FROMINT(401) },
  { FIXED_FROMINT(553),  FIXED_FROMINT(834) },
  { FIXED_FROMINT(209),  FIXED_FROMINT(653) }
};

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void ScrollMap();
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    
    
    // Initialize the Map-System. If you do not initialize
    // the Map-System before calling any Map related function,
    // the program behaviour is undefined.
    hel_MapInit(g_MapSystemBuffer);
    
    ham_SetBgMode(0);
    
    ham_InitText(1);
    ham_SetTextCol(3, 0);
    
    
    // Load Tileset into VideoRAM
    ham_bg[0].ti = ham_InitTileSet(ResData(RES_SETPOS_RAW), RES_SETPOS_RAW_SIZE16, COLORS_16, TRUE);
    
    // Create a map for background 0
    hel_MapCreate(0,        // Background number
                  2248/8,   // width in tiles
                  1184/8,   // height in tiles
                  ResData(RES_SETPOS_MAP),   // Pointer to source MapData
                  sizeof(u16),                  // DataTypeSize of one element from Source MapData
                  MAP_FLAGS_DEFAULT);           // Flags to control map behaviour
    
    // turn off boundschecking, which
    // results into map wraparound
    hel_MapSetBounds(0, 0, 0, 0, 0, 0);
    
    // Load Background Palette
    hel_PalBgLoad256(ResData(RES_BG_PAL));
    
    // Turn on Background
    ham_InitBg(0, TRUE, 1, FALSE);
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    // draw some text
    ham_DrawText(1, 17, "-A- for random map marker");
    
    
    // Infinite loop to keep
    // the application alive
    for(;;)
    {
      hel_PadCapture();
      
      if(hel_PadQuery()->Pressed.A)
      {
        // get a random map-marker
        TPointSfp32 *pPos = (TPointSfp32*)&MAPMARKER[rand()%MAPMARKER_COUNT];
        
        // set it to the position
        hel_MapSetPosition(0, pPos->X, pPos->Y);
        
        // output the position
        ham_DrawText(1, 18, "X=%d, Y=%d     ", FIXED_TOINT(pPos->X), FIXED_TOINT(pPos->Y));
      }
      
      ScrollMap();
          
      hel_SwiVBlankIntrWait();
    }
    
    return 0;
}


// -----------------------------------------------------------------------------
// Queries the Pad and scrolls the map depending on the pad input
// -----------------------------------------------------------------------------
void ScrollMap()
{
   #define SCROLL_SPEED_X     FIXED_FROMFLOAT(1.0f)
   #define SCROLL_SPEED_Y     FIXED_FROMFLOAT(1.0f)

   TPointSfp32 ScrollBy={0, 0};

   // Query Pad to see in what direction
   // the map should be scrolled
   if(hel_PadQuery()->Held.Left)
   {
      ScrollBy.X -= SCROLL_SPEED_X;
   }
   else
   if(hel_PadQuery()->Held.Right)
   {
      ScrollBy.X += SCROLL_SPEED_X;
   }

   if(hel_PadQuery()->Held.Down)
   {
      ScrollBy.Y += SCROLL_SPEED_Y;
   }
   else
   if(hel_PadQuery()->Held.Up)
   {
      ScrollBy.Y -= SCROLL_SPEED_Y;
   }

   // Now Scroll the map!
   hel_MapScroll(0, ScrollBy.X, ScrollBy.Y);
   
   #undef SCROLL_SPEED_X
   #undef SCROLL_SPEED_Y
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	hel_IntrAcknowledge(INT_TYPE_VBL);
}

