// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to perform flipping using the Object System.
//                The source graphics are flipped, so you only see the
//                text right when the objects have flipping activated.
//
//                If you use HEL's Object-System, you must not use any
//                object related function from HAM anymore. If you mix calls
//                to object function from both API's, HAM and HEL, the output 
//                will probably be something unexpected.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------

// Reserve memory for the Obj-System. This buffer
// is used to manage internal states of the Obj-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void CreateObjects(void);
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);
    
    // Initialize Object-System
    hel_ObjInit(g_ObjSystemBuffer);
    
    // Create flipped objects
    CreateObjects();
                     
    // load object palette
    hel_PalObjLoad256(ResData16(RES_OBJECT_PAL));
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    
    for(;;)
    {
        hel_SwiVBlankIntrWait();
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// Create the three flipped objects
// -----------------------------------------------------------------------------
void CreateObjects(void)
{
  THandle hObj;
  
  
  // Create a horizontally flipped object!
  hObj = hel_ObjCreate(
                   ResData(RES_HFLIP_RAW), // Pointer to source graphic
                   OBJ_SHAPE_SQUARE,       // Obj Shape
                   3,                      // Obj Size, 3 means 64x64 pixels, if Shape is set to SQUARE
                   OBJ_MODE_NORMAL,        // Obj Mode
                   COLORS_256,             // Use 256 color mode
                   0,                      // Palette number. Only neccessary in 16 color mode
                   FALSE,                  // Don't use mosaic
                   FALSE,                  // Don't flip the sprite horizontally
                   FALSE,                  // Don't flip the object vertically
                   0,                      // Priority against background. 0=higest
                   FALSE,                  // Don't make the object double-sized
                   10,                     // Destination horzintal screen coordinate in pixels
                   30                      // Destination vertical screen coordinate in pixels
                );
  
  hel_ObjSetHFlip(hObj, TRUE);
   
   
   
  // Create a vertically flipped object!
  hObj = hel_ObjCreate(
                   ResData(RES_VFLIP_RAW), // Pointer to source graphic
                   OBJ_SHAPE_SQUARE,       // Obj Shape
                   3,                      // Obj Size, 3 means 64x64 pixels, if Shape is set to SQUARE
                   OBJ_MODE_NORMAL,        // Obj Mode
                   COLORS_256,             // Use 256 color mode
                   0,                      // Palette number. Only neccessary in 16 color mode
                   FALSE,                  // Don't use mosaic
                   FALSE,                  // Don't flip the sprite horizontally
                   FALSE,                  // Don't flip the object vertically
                   0,                      // Priority against background. 0=higest
                   FALSE,                  // Don't make the object double-sized
                   85,                     // Destination horzintal screen coordinate in pixels
                   50                      // Destination vertical screen coordinate in pixels
                );
                
  hel_ObjSetVFlip(hObj, TRUE);
  
  
  
  // Create a horizontally and vertically flipped object!
  // This time we do not use the HFlip and VFlip
  // argument when creating the object, we use the
  // hel_ObjSetHFlip and hel_ObjSetVFlip functions instead.
  hObj = hel_ObjCreate(
                   ResData(RES_HVFLIP_RAW),// Pointer to source graphic
                   OBJ_SHAPE_SQUARE,       // Obj Shape
                   3,                      // Obj Size, 3 means 64x64 pixels, if Shape is set to SQUARE
                   OBJ_MODE_NORMAL,        // Obj Mode
                   COLORS_256,             // Use 256 color mode
                   0,                      // Palette number. Only neccessary in 16 color mode
                   FALSE,                  // Don't use mosaic
                   TRUE,                   // Don't flip the sprite horizontally
                   TRUE,                   // Don't flip the object vertically
                   0,                      // Priority against background. 0=higest
                   FALSE,                  // Don't make the object double-sized
                   160,                    // Destination horzintal screen coordinate in pixels
                   70                      // Destination vertical screen coordinate in pixels
                );
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
   // Transmit object attributes to hardware
   hel_ObjTransmit();

   hel_IntrAcknowledge(INT_TYPE_VBL);
}

