// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use the Object Rotation/Scaling System.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define OBJ_COUNT 6

// -----------------------------------------------------------------------------
// Structures
// -----------------------------------------------------------------------------
typedef struct TRotationObject
{
  u8 ObjNo;       // Handle of Object
  THandle RotNo;  // Handle of Rotation Set
  u16 Angle;      // Helper variable for angle
  s16 Zoom;       // Helper variable for zoom
}TRotationObject;

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
TRotationObject ATTR_EWRAM g_Obj[OBJ_COUNT];
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];

// Reserve memory for the Object-Rotation/Scaling-System. This buffer
// is used to manage internal states of the Object-Rotation/Scaling-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjRotScaleSystemBuffer[HEL_SUBSYSTEM_OBJROTSCALE_REQUIREDMEMORY];



// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void InitObjects(void);
void UpdateObjects(void);
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);
    
    // Initialize Object-System
    hel_ObjInit(g_ObjSystemBuffer);
    
    // Init the Object Rotation/Scaling System
    hel_ObjRotScaleInit(g_ObjRotScaleSystemBuffer);
    
    // init objects
    InitObjects();
    
    // load object palette
    hel_PalObjLoad256(ResData16(RES_OBJECT_PAL));
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    
    for(;;)
    {
        // update object graphics
        UpdateObjects();

        hel_SwiVBlankIntrWait();
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// Create OBJ_COUNT objects and rotation/scaling sets
// -----------------------------------------------------------------------------
void InitObjects(void)
{
  int i;
  
  for(i=0; i<OBJ_COUNT; i++)
  {
    // Create Object
    g_Obj[i].ObjNo = hel_ObjCreate256
                      (
                        ResData(RES_HEART_RAW), // Sourcegraphic
                        OBJ_SHAPE_SQUARE,       // Object Shape
                        1,                      // Object Size
                        OBJ_MODE_NORMAL,        // Object Mode
                        24 + i*30,              // X
                        60                      // Y
                      );
                        
    // Set random Angle and Zoom helper member variables
    g_Obj[i].Angle = i*(360/OBJ_COUNT);
    g_Obj[i].Zoom  = i*20;

    // Create Rotation/Scaling Set
    g_Obj[i].RotNo = hel_ObjRotScaleCreate
                      (
                        0,                    // Angle in degree
                        0x100,                // ZoomX (0x100 = no zoom)
                        0x100                 // ZoomY (0x100 = no zoom)
                      );

    // Enable Rotation/Scaling for the Object
    hel_ObjSetRotScale(g_Obj[i].ObjNo, TRUE);

    // Enable Double Size
    hel_ObjSetDblSize(g_Obj[i].ObjNo, TRUE);
    
    // Select the Rotation/Scaling Set for the Object
    hel_ObjSelectRotScale(g_Obj[i].ObjNo, g_Obj[i].RotNo);
  }
}


// -----------------------------------------------------------------------------
// Update Objects, load new graphic into Vram.
// -----------------------------------------------------------------------------
void UpdateObjects(void)
{
  int i;

  // reload object graphic!
  for(i=0; i<OBJ_COUNT; i++)
  {
    // Update Angle and Zoom helpers
    g_Obj[i].Angle += 1;
    g_Obj[i].Zoom  += 2;
    
    // Update Rotation and Scaling
    hel_ObjRotScaleUpdate
      (
        g_Obj[i].RotNo,
        (g_Obj[i].Angle%359),
        0x190 - ABS(SIN(g_Obj[i].Zoom)),
        0x190 - ABS(COS(g_Obj[i].Zoom*2))
      );
  }
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
   // Transmit object attributes to hardware
   hel_ObjTransmit();
  
   // Transmit Object Rotation/Scaling Attributes to hardware
   hel_ObjRotScaleTransmit();

   hel_IntrAcknowledge(INT_TYPE_VBL);
}

